/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  DevicePathToTextBBTestCoverage.c

Abstract:

  Interface Function Test Cases of Device Path To Text Protocol

--*/

#include "DevicePathToTextBBTestMain.h"

#include EFI_GUID_DEFINITION (PcAnsi)

extern EFI_GUID gEfiDevicePathFromTextProtocolGuid;
extern EFI_GUID gEfiDevicePathUtilitiesProtocolGuid;

#define MaxDevicePathStrLen 200

#define IS_HYPHEN(a)         ((a) == L'-')
#define IS_NULL(a)           ((a) == L'\0')

STATIC
EFI_STATUS 
StrToBuf (
  OUT UINT8    *Buf,
  IN  UINTN    BufferLength,
  IN  CHAR16   *Str
  )
{
  UINTN       Index;
  UINTN       StrLength;
  UINT8       Digit;
  UINT8       Byte;

  //
  // Two hex char make up one byte
  //
  StrLength = BufferLength * sizeof (CHAR16);

  for(Index = 0; Index < StrLength; Index++, Str++) {

    IsHexDigit (&Digit, *Str);

    //
    // For odd charaters, write the upper nibble for each buffer byte,
    // and for even characters, the lower nibble.
    //
    if ((Index & 1) == 0) {
      Byte = Digit << 4;
    } else {
      Byte = Buf[Index / 2];
      Byte &= 0xF0;
      Byte |= Digit;
    }

    Buf[Index / 2] = Byte;
  }

  return EFI_SUCCESS;
}

STATIC
EFI_STATUS
StrToGuid (
  IN  CHAR16   *Str,
  OUT EFI_GUID *Guid
  )
{
  UINTN       BufferLength;
  UINTN       ConvertedStrLen;
  EFI_STATUS  Status;

  BufferLength = sizeof (Guid->Data1);
  Status = HexStringToBuf ((UINT8 *) &Guid->Data1, &BufferLength, Str, &ConvertedStrLen);
  if (EFI_ERROR (Status)) {
    return Status;
  }
  Str += ConvertedStrLen;
  if (IS_HYPHEN (*Str)) {
    Str++;   
  } else {
    return EFI_UNSUPPORTED;
  }

  BufferLength = sizeof (Guid->Data2);
  Status = HexStringToBuf ((UINT8 *) &Guid->Data2, &BufferLength, Str, &ConvertedStrLen);
  if (EFI_ERROR (Status)) {
    return Status;
  }
  Str += ConvertedStrLen;
  if (IS_HYPHEN (*Str)) {
    Str++;
  } else {
    return EFI_UNSUPPORTED;
  }

  BufferLength = sizeof (Guid->Data3);
  Status = HexStringToBuf ((UINT8 *) &Guid->Data3, &BufferLength, Str, &ConvertedStrLen);
  if (EFI_ERROR (Status)) {
    return Status;
  }
  Str += ConvertedStrLen;
  if (IS_HYPHEN (*Str)) {
    Str++;
  } else {
    return EFI_UNSUPPORTED;
  }

  StrToBuf (&Guid->Data4[0], 2, Str);
  //
  // Skip 2 byte hex chars
  //
  Str += 2 * 2;

  if (IS_HYPHEN (*Str)) {
    Str++;
  } else {
    return EFI_UNSUPPORTED;
  }
  StrToBuf (&Guid->Data4[2], 6, Str);

  return EFI_SUCCESS;
}

STATIC
UINT32
ConvertEisaIDFromStr (
  IN CHAR16             *Str
  )
{
  if ((Str[0] == L'P') && (Str[1] == L'N') && (Str[2] == L'P')) {
    return EISA_PNP_ID(xtoi(Str + 3));
  }
  return (UINT32)xtoi(Str);
}

STATIC
VOID
ConvertStrToIPv4Addr (
  IN OUT CHAR16           *Str,
  OUT    EFI_IPv4_ADDRESS *IPv4Addr
  )
{
  UINTN   Index;
  CHAR16  *AddrStart;
  CHAR16  *AddrEnd;

  AddrStart = (CHAR16 *)AllocateCopyPool(StrLen(Str) * 2 + 2, Str);
  for (Index = 0; Index < 4; Index++) {
    AddrEnd   = StrChr(AddrStart, L'.');
    if (AddrEnd != NULL) {
      *AddrEnd  = L'\0';
    }
    IPv4Addr->Addr[Index] = (UINT8) Atoi (AddrStart);
    AddrStart = AddrEnd + 1;
  }
}

STATIC
VOID
ConvertStrToIPv6Addr (
  IN OUT CHAR16           *Str,
  OUT    EFI_IPv6_ADDRESS *IPv6Addr
  )
{
  UINTN   Index;
  CHAR16  *AddrStart;
  CHAR16  *AddrEnd;
  UINT16  Data;

  AddrStart = (CHAR16 *)AllocateCopyPool(StrLen(Str) * 2 + 2, Str);
  for (Index = 0; Index < 8; Index++) {
    AddrEnd   = StrChr(AddrStart, L':');
    if (AddrEnd != NULL) {
      *AddrEnd  = L'\0';
    }
    Data = (UINT16) xtoi (AddrStart);
    IPv6Addr->Addr[Index * 2]     = (UINT8) (Data >> 8);
    IPv6Addr->Addr[Index * 2 + 1] = (UINT8) (Data & 0xff);
    AddrStart = AddrEnd + 1;
  }
}

//
// TDS 3.7.1
//
EFI_STATUS
DevicePathToTextConvertDeviceNodeToTextCoverageTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STATUS                          Status;
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL  *StandardLib;
  EFI_TEST_ASSERTION                  AssertionType;
  EFI_DEVICE_PATH_TO_TEXT_PROTOCOL    *DevicePathToText;
  EFI_DEVICE_PATH_UTILITIES_PROTOCOL  *DevicePathUtilities;
  EFI_DEVICE_PATH                     *pDeviceNode1;
  EFI_DEVICE_PATH                     *pDeviceNode2;
  CHAR16                              *Text;
  UINTN                               Length;
  EFI_GUID                            TmpGuid;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                  SupportHandle,
                  &gEfiStandardTestLibraryGuid,
                  &StandardLib
                  );

  if (EFI_ERROR (Status)) {
    return Status;
  }

  Status = gtBS->LocateProtocol (
                  &gEfiDevicePathUtilitiesProtocolGuid,
                  NULL,
                  &DevicePathUtilities
                  );
  if (EFI_ERROR (Status)) {
    return Status;
  }

  DevicePathToText = (EFI_DEVICE_PATH_TO_TEXT_PROTOCOL *) ClientInterface;

  //
  // PcCard(0x2A)
  // 
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (1, 2, 5);
  ((PCCARD_DEVICE_PATH *)pDeviceNode1)->FunctionNumber = 0x2A;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid089,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: PcCard(0x2A)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // MemoryMapped(14,0x123456789ABCDEF,0xFEDCBA9876543210)
  // 
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x1, 0x3, 24);
  ((MEMMAP_DEVICE_PATH *)pDeviceNode1)->MemoryType      = 14;
  ((MEMMAP_DEVICE_PATH *)pDeviceNode1)->StartingAddress = 0x123456789ABCDEF;
  ((MEMMAP_DEVICE_PATH *)pDeviceNode1)->EndingAddress   = 0xFEDCBA9876543210;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid090,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: MemoryMapped(14,0x123456789ABCDEF,0xFEDCBA9876543210)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // VenHw(5AF6C71E-1261-4637-9838-C4E9913D1DBB,0123456789ABCDEF)
  //
  Length = (EfiStrLen (L"0123456789ABCDEF") + 1) / 2;
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x1, 0x4, 20 + (UINT16)Length);
  StrToGuid (L"5AF6C71E-1261-4637-9838-C4E9913D1DBB", &((VENDOR_DEVICE_PATH *)pDeviceNode1)->Guid);
  StrToBuf (((UINT8 *) pDeviceNode1) + 20, Length, L"0123456789ABCDEF");

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid091,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: VenHw(5AF6C71E-1261-4637-9838-C4E9913D1DBB,0123456789ABCDEF)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Ctrl(0x1234ABCD)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x1, 0x5, 0x8);
  ((CONTROLLER_DEVICE_PATH *)pDeviceNode1)->Controller = 0x1234ABCD;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid092,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Ctrl(0x1234ABCD)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // AcpiEx(PNP0A03,PNP0A03,0x6789A,,,)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x2, 0x2, 19);
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HID = ConvertEisaIDFromStr(L"PNP0A03");
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->CID = ConvertEisaIDFromStr(L"PNP0A03");
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->UID = 0x6789A;
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[0] = '\0';
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[1] = '\0';
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[2] = '\0';

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid093,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: AcpiEx(PNP0A03,PNP0A03,0x6789A,,,)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // AcpiExp(PNP0A03,PNP0A03,)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x2, 0x2, 19);
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HID = ConvertEisaIDFromStr(L"PNP0A03");
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->CID = ConvertEisaIDFromStr(L"PNP0A03");
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->UID = 0;
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[0] = '\0';
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[1] = '\0';
  ((ACPI_EXTENDED_HID_DEVICE_PATH_WITH_STR *)pDeviceNode1)->HidUidCidStr[2] = '\0';

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid094,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: AcpiExp(PNP0A03,PNP0A03,)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }


  //
  // Scsi(1234,0xABCD)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x2, 0x8);
  ((SCSI_DEVICE_PATH *)pDeviceNode1)->Pun = 1234;
  ((SCSI_DEVICE_PATH *)pDeviceNode1)->Lun = 0xABCD;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid095,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Scsi(1234,0xABCD)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Fibre(0x123456789ABCDEF0,0xABCDEF1234567890)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x3, 24);
  ((FIBRECHANNEL_DEVICE_PATH *)pDeviceNode1)->WWN      = 0x123456789ABCDEF0;
  ((FIBRECHANNEL_DEVICE_PATH *)pDeviceNode1)->Lun      = 0xABCDEF1234567890;
  ((FIBRECHANNEL_DEVICE_PATH *)pDeviceNode1)->Reserved = 0;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid096,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Fibre(0x123456789ABCDEF0,0xABCDEF1234567890)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // USB(12,0xAB)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x5, 6);
  ((USB_DEVICE_PATH *)pDeviceNode1)->ParentPortNumber = 12;
  ((USB_DEVICE_PATH *)pDeviceNode1)->InterfaceNumber  = 0xAB;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid098,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: USB(12,0xAB)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // I2O(0x1234ABCD)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x6, 8);
  ((I2O_DEVICE_PATH *)pDeviceNode1)->Tid = 0x1234ABCD;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid099,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: I2O(0x1234ABCD)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Infiniband(10,A1ED07B9-5BDC-4741-ADCF-CA1E8681DD1A,1234,5678,0xABCD)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x9, 48);
  ((INFINIBAND_DEVICE_PATH *)pDeviceNode1)->ResourceFlags = 10;
  StrToGuid (L"A1ED07B9-5BDC-4741-ADCF-CA1E8681DD1A", &TmpGuid);
  CopyMem (&((INFINIBAND_DEVICE_PATH *)pDeviceNode1)->PortGid, &TmpGuid, sizeof (EFI_GUID));
  ((INFINIBAND_DEVICE_PATH *)pDeviceNode1)->ServiceId     = 1234;
  ((INFINIBAND_DEVICE_PATH *)pDeviceNode1)->TargetPortId  = 5678;
  ((INFINIBAND_DEVICE_PATH *)pDeviceNode1)->DeviceId      = 0xABCD;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid100,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Infiniband(10,A1ED07B9-5BDC-4741-ADCF-CA1E8681DD1A,1234,5678,0xABCD)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // VenPcAnsi()
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xa, 20);
  ((VENDOR_DEVICE_PATH *)pDeviceNode1)->Guid = gEfiPcAnsiGuid;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid101,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: VenPcAnsi()",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // UartFlowCtrl(Hardware)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xa, 24);
  ((UART_FLOW_CONTROL_DEVICE_PATH *)pDeviceNode1)->Guid           = gEfiPcAnsiGuid;
  ((UART_FLOW_CONTROL_DEVICE_PATH *)pDeviceNode1)->FlowControlMap = 1;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid102,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: UartFlowCtrl(Hardware)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // MAC(12345678ABCD12345678ABCD12345678ABCD12345678ABCD12345678ABCD1234,2)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xa, 37);
  StrToBuf (&((MAC_ADDR_DEVICE_PATH *)pDeviceNode1)->MacAddress.Addr[0], sizeof (EFI_MAC_ADDRESS), L"12345678ABCD12345678ABCD12345678ABCD12345678ABCD12345678ABCD1234");
  ((MAC_ADDR_DEVICE_PATH *)pDeviceNode1)->IfType = 2;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid105,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: MAC(12345678ABCD12345678ABCD12345678ABCD12345678ABCD12345678ABCD1234,2)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // IPv4(12.34.56.78,TCP,Static,123.45.67.89)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xc, 19);
  ConvertStrToIPv4Addr (L"12.34.56.78", &((IPv4_DEVICE_PATH *)pDeviceNode1)->RemoteIpAddress);
  ((IPv4_DEVICE_PATH *)pDeviceNode1)->Protocol        = 1;
  ((IPv4_DEVICE_PATH *)pDeviceNode1)->StaticIpAddress = TRUE;
  ((IPv4_DEVICE_PATH *)pDeviceNode1)->LocalPort       = 0;
  ((IPv4_DEVICE_PATH *)pDeviceNode1)->RemotePort      = 0;
  ConvertStrToIPv4Addr (L"123.45.67.89", &((IPv4_DEVICE_PATH *)pDeviceNode1)->LocalIpAddress);

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid106,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: IPv4(12.34.56.78,TCP,Static,123.45.67.89)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // IPv6(1234:5678:ABCD:1234:5678:ABCD:1234:5678,UDP,DHCP,5678:ABCD:1234:5678:ABCD:1234:5678:ABCD)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xd, 43);
  ConvertStrToIPv6Addr (L"1234:5678:ABCD:1234:5678:ABCD:1234:5678", &((IPv6_DEVICE_PATH *)pDeviceNode1)->RemoteIpAddress);
  ((IPv6_DEVICE_PATH *)pDeviceNode1)->Protocol        = 0;
  ((IPv6_DEVICE_PATH *)pDeviceNode1)->StaticIpAddress = FALSE;
  ConvertStrToIPv6Addr (L"5678:ABCD:1234:5678:ABCD:1234:5678:ABCD", &((IPv6_DEVICE_PATH *)pDeviceNode1)->LocalIpAddress);

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(L"IPv6(1234:5678:ABCD:1234:5678:ABCD:1234:5678,UDP,DHCP,5678:ABCD:1234:5678:ABCD:1234:5678:ABCD)");

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid107,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: IPv6(1234:5678:ABCD:1234:5678:ABCD:1234:5678,UDP,DHCP,5678:ABCD:1234:5678:ABCD:1234:5678:ABCD)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Uart(115200,8,O,1.5)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xe, 19);
  ((UART_DEVICE_PATH *)pDeviceNode1)->Reserved = 0;
  ((UART_DEVICE_PATH *)pDeviceNode1)->BaudRate = 115200;
  ((UART_DEVICE_PATH *)pDeviceNode1)->DataBits = 8;
  ((UART_DEVICE_PATH *)pDeviceNode1)->Parity   = 3;
  ((UART_DEVICE_PATH *)pDeviceNode1)->StopBits = 2;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid108,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Uart(115200,8,O,1.5)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // UsbClass(1234,5678,14,1,0)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0xf, 11);
  ((USB_CLASS_DEVICE_PATH *)pDeviceNode1)->VendorId       = 1234;
  ((USB_CLASS_DEVICE_PATH *)pDeviceNode1)->ProductId      = 5678;
  ((USB_CLASS_DEVICE_PATH *)pDeviceNode1)->DeviceClass    = 14;
  ((USB_CLASS_DEVICE_PATH *)pDeviceNode1)->DeviceSubClass = 1;
  ((USB_CLASS_DEVICE_PATH *)pDeviceNode1)->DeviceProtocol = 0;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid109,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: UsbClass(1234,5678,14,1,0)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Unit(0xAB)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x3, 0x11, 5);
  ((DEVICE_LOGICAL_UNIT_DEVICE_PATH *)pDeviceNode1)->Lun  = 0xAB;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid113,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Unit(0xAB)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // HD(6,GPT,E8AAED38-1815-4E4F-BCB5-2E3DBD160C9C,12345678,0xABCD1234)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x4, 0x1, 42);
  ((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->PartitionNumber = 6;
  ((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->MBRType         = 2;
  ((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->PartitionStart  = 12345678;
  ((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->PartitionSize   = 0xABCD1234;
  ((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->SignatureType   = 2;
  StrToGuid (L"E8AAED38-1815-4E4F-BCB5-2E3DBD160C9C", &TmpGuid);
  CopyMem (&((HARDDRIVE_DEVICE_PATH *)pDeviceNode1)->Signature, &TmpGuid, sizeof (EFI_GUID));

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid115,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: HD(6,GPT,E8AAED38-1815-4E4F-BCB5-2E3DBD160C9C,12345678,0xABCD1234)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // CDROM(12340000,12345678,0xABCD1234)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x4, 0x2, 24);
  ((CDROM_DEVICE_PATH *)pDeviceNode1)->BootEntry      = 12340000;
  ((CDROM_DEVICE_PATH *)pDeviceNode1)->PartitionStart = 12345678;
  ((CDROM_DEVICE_PATH *)pDeviceNode1)->PartitionSize  = 0xABCD1234;

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid117,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: CDROM(12340000,12345678,0xABCD1234)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  //
  // Media(BA3A77E6-39A2-4375-A39A-108BFFCCE1AA)
  //
  pDeviceNode1 = DevicePathUtilities->CreateDeviceNode (0x4, 0x5, 20);
  StrToGuid (L"BA3A77E6-39A2-4375-A39A-108BFFCCE1AA", &((MEDIA_PROTOCOL_DEVICE_PATH *)pDeviceNode1)->Protocol);

  Text = DevicePathToText->ConvertDeviceNodeToText (pDeviceNode1, FALSE, FALSE);
  pDeviceNode2 = SctConvertTextToDeviceNode(Text);

  if ((pDeviceNode2 != NULL) && (EfiCompareMem (pDeviceNode2, pDeviceNode1, DevicePathNodeLength(pDeviceNode1)) == 0)) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }

  StandardLib->RecordAssertion (
                StandardLib,
                AssertionType,
                gDevicePathBBTestFunctionAssertionGuid119,
                L"EFI_DEVICE_PATH_FROM_TEXT_PROTOCOL - ConvertDeviceNodeToText must correctly recover the converting ConvertTextToDeviceNode has acted on the device node string",
                L"%a:%d: Convert result: %s - Expected: Media(BA3A77E6-39A2-4375-A39A-108BFFCCE1AA)",
                __FILE__,
                (UINTN)__LINE__,
                Text
                );
  if (pDeviceNode1 != NULL) {
    FreePool (pDeviceNode1);
  }
  if (pDeviceNode2 != NULL) {
    FreePool (pDeviceNode2);
  }
  if (Text != NULL) {
    FreePool (Text);
  }

  return EFI_SUCCESS;
}

